//=================================================================================
// Copyright (C) 2004-2006 Ice Tea Group, LLC
// 
// By using this code you have agreed to the terms and conditions set forth
// in the "PPJ Framework EULA" and "PPJ Framework Source EULA" licenses that
// have been delivered to you. In case you do not have these two documents
// you are hereby notified that you are not permitted to use this code.
// Please contact us at info@iceteagroup.com for further assistance.
// EULA stands for End User License Agreement.
// 
// ICE TEA GROUP LLC SHALL IN NO EVENT BE LIABLE FOR ANY DAMAGES WHATSOEVER
// (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS
// INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER LOSS OF ANY KIND)
// ARISING OUT OF THE USE OR INABILITY TO USE THE GENERATED CODE, WHETHER
// DIRECT, INDIRECT, INCIDENTAL, CONSEQUENTIAL, SPECIAL OR OTHERWISE, REGARDLESS
// OF THE FORM OF ACTION, EVEN IF ICE TEA GROUP LLC HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGES.
//=================================================================================

using System;
using System.Text;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Text;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.ComponentModel.Design.Serialization;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using PPJ.Runtime;
using PPJ.Runtime.Windows;
using PPJ.Runtime.Windows.QO;
using PPJ.Runtime.Windows.SDK;
using System.Windows.Forms.VisualStyles;

namespace WindowsApplication14
{
	public class SalQuickTabsDraw
	{
		public static void Install(SalQuickTabs tabs)
		{
			tabs.DrawControl += new PaintEventHandler(tabs_DrawControl);
		}

		static void tabs_DrawControl(object sender, PaintEventArgs e)
		{
			DrawControl((TabControl)sender, e.Graphics);
		}

		private static void DrawControl(TabControl tab, Graphics g)
		{
			if (!tab.Visible) return;

			VisualStyleRenderer render = new VisualStyleRenderer(VisualStyleElement.Tab.Pane.Normal);
			Rectangle TabPageArea = tab.ClientRectangle;
			Rectangle TabArea = tab.ClientRectangle;
			switch (tab.Alignment)
			{
				case TabAlignment.Top:
					TabArea = tab.ClientRectangle;
					TabArea.Height = tab.DisplayRectangle.Y - 2;

					TabPageArea = tab.ClientRectangle;
					TabPageArea.Offset(0, tab.DisplayRectangle.Y - 2);
					TabPageArea.Height -= tab.DisplayRectangle.Y - 2;
					break;
				case TabAlignment.Left:
					TabArea = tab.ClientRectangle;
					TabArea.Width = tab.DisplayRectangle.X - 2;

					TabPageArea = tab.ClientRectangle;
					TabPageArea.Offset(tab.DisplayRectangle.X - 1, 0);
					TabPageArea.Width = tab.DisplayRectangle.Width + 6;
					break;
				case TabAlignment.Right:
					TabArea = tab.ClientRectangle;
					TabArea.Width -= (tab.DisplayRectangle.Width + tab.DisplayRectangle.X);
					TabArea.X = (tab.DisplayRectangle.Width + tab.DisplayRectangle.X + 2);

					TabPageArea = tab.DisplayRectangle;
					TabPageArea.Inflate(4, 4);
					break;
				case TabAlignment.Bottom:
					TabArea = tab.ClientRectangle;
					TabArea.Height -= (tab.DisplayRectangle.Height + tab.DisplayRectangle.Y);
					TabArea.Y = (tab.DisplayRectangle.Height + tab.DisplayRectangle.Y);
					break;

			}


			render.DrawBackground(g, TabPageArea);
			render.DrawParentBackground(g, TabArea, tab);

			for (int i = 0; i < tab.TabCount; i++)
			{
				if (i != tab.SelectedIndex)
					DrawTab(g, i, tab);
			}

			DrawTab(g, tab.SelectedIndex, tab);
		}

		private static void DrawTab(Graphics g, int nIndex, TabControl tab)
		{
			TabPage tabPage = tab.TabPages[nIndex];
			bool disablePage = !((SalQuickTabs.SalTabPage)tabPage).Enabled;
			Brush br = new SolidBrush(tabPage.ForeColor);

			Rectangle recBounds = tab.GetTabRect(nIndex);

			Rectangle tabTextArea = tab.GetTabRect(nIndex);
			Rectangle rubberArea = tab.GetTabRect(nIndex);
			Boolean bSelected = (tab.SelectedIndex == nIndex);
			VisualStyleRenderer render = new VisualStyleRenderer(VisualStyleElement.Tab.TabItem.Normal);
			Point mousePoint = tab.PointToClient(Control.MousePosition);
			bool myMouseHover = recBounds.Contains(mousePoint);

			if (bSelected)
			{
				tabTextArea.Y -= 1;
				rubberArea.Inflate(-1, -1);
				rubberArea.Width -= 1;
				rubberArea.Height -= 1;
				recBounds.Inflate(2, 2);
				if (nIndex == 0 && tab.TabCount > 1)
				{
					render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItemLeftEdge.Pressed);
				}
				else if (nIndex == tab.TabCount - 1)
				{
					render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItem.Pressed);
				}
				else
				{
					render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItem.Pressed);
				}
				render.DrawBackground(g, recBounds);
				render.DrawEdge(g, recBounds, Edges.Diagonal, EdgeStyle.Sunken, EdgeEffects.Flat);
				Pen pen = new Pen(br);
				pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;
				g.DrawRectangle(pen, rubberArea);
			}
			else
			{
				tabTextArea.Y += 1;
				if (myMouseHover && !disablePage)
				{
					render = new VisualStyleRenderer(VisualStyleElement.Tab.TabItem.Hot);
					myMouseHover = false;
					render.DrawEdge(g, recBounds, Edges.Diagonal, EdgeStyle.Sunken, EdgeEffects.Flat);
					render.DrawBackground(g, recBounds);
				}
				else
				{
					if (nIndex == 0 && tab.TabCount > 1)
					{
						if (disablePage)
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItemLeftEdge.Disabled);
						else
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItemLeftEdge.Normal);
					}
					else if (nIndex == tab.TabCount - 1)
					{
						if (disablePage)
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItemRightEdge.Disabled);
						else
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItemRightEdge.Normal);
					}
					else
					{
						if (disablePage)
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItem.Disabled);
						else
							render = new VisualStyleRenderer(VisualStyleElement.Tab.TopTabItem.Normal);
					}
					render.DrawBackground(g, recBounds);
					render.DrawEdge(g, recBounds, Edges.Diagonal, EdgeStyle.Sunken, EdgeEffects.Flat);
				}
			}

			StringFormat stringFormat = new StringFormat();
			stringFormat.Alignment = StringAlignment.Center;
			stringFormat.LineAlignment = StringAlignment.Center;
			stringFormat.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show;

			if (tabPage.ImageIndex >= 0 && tab.ImageList != null && tab.ImageList.Images[tabPage.ImageIndex] != null)
			{
				int nLeftMargin = 8;
				int nRightMargin = 2;
				Image img = tab.ImageList.Images[tabPage.ImageIndex];
				Rectangle rimage = new Rectangle(recBounds.X + nLeftMargin, recBounds.Y + 1, img.Width, img.Height);
				int nAdj = nLeftMargin + img.Width + nRightMargin;
				rimage.Y += (recBounds.Height - img.Height) / 2;
				tabTextArea.X += nAdj;
				tabTextArea.Width -= nAdj;
				g.DrawImage(img, rimage);
				stringFormat.Alignment = StringAlignment.Near;
			}

			if (disablePage)
			{
				tabTextArea.Offset(1, 1);
				g.DrawString(tabPage.Text, tab.Font, SystemBrushes.ControlLightLight, tabTextArea, stringFormat);
				tabTextArea.Offset(-1, -1);
				g.DrawString(tabPage.Text, tab.Font, SystemBrushes.ControlDark, tabTextArea, stringFormat);
			}
			else
			{
				g.DrawString(tabPage.Text, tab.Font, br, tabTextArea, stringFormat);
			}
		}
	}
}
